------------------------------------------------------------------------
--  LCD driver
--  purpose: to initialize the LCD and wait for the user
--  to input the characters.
------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity lcddriver is
	port(
        clock: in std_logic;
        reset: in std_logic;
	message: in std_logic_vector (3 downto 0); 	-- input from outside such as a keypad
	lcd_select: out std_logic; 			-- lcd register select signal   modo
	lcd_data: out std_logic_vector(7 downto 0);	-- saida
	lcd_rw: out std_logic;				-- rw
	lcd_enable: out std_logic;			-- hab
	enable: in std_logic 				-- detect if there is any incoming message
	);
end lcddriver;

-- Architecture Description

architecture Display of lcddriver is    
	TYPE STATE_TYPE IS (initial, display, entrymode, clear, waiting, verify,putchar, homecursor);
	SIGNAL state,next_state: STATE_TYPE;

	-- Clear screen.
	constant CLR:   std_logic_vector(7 downto 0) := "00000001";

	-- Display ON, with cursor.
	constant DON:   std_logic_vector(7 downto 0) := "00001110";

	-- Set Entry Mode to increment cursor automatically after each character
	-- is displayed.
	constant SEM:   std_logic_vector(7 downto 0) := "00000110";

	-- Home cursor
	constant HOME:  std_logic_vector(7 downto 0) := "00000010";

	-- Function set for 8-bit data transfer and 2-line display
	constant SET:   std_logic_vector(7 downto 0) := "00111000";

	-- The numbers and symbols
	constant zero:	std_logic_vector(7 downto 0) := "00110000";
	constant one:	std_logic_vector(7 downto 0) := "00110001";
	constant two:	std_logic_vector(7 downto 0) := "00110010";
	constant three:	std_logic_vector(7 downto 0) := "00110011";
	constant four:	std_logic_vector(7 downto 0) := "00110100";
	constant five:	std_logic_vector(7 downto 0) := "00110101";
	constant six:	std_logic_vector(7 downto 0) := "00110110";
	constant seven:	std_logic_vector(7 downto 0) := "00110111";
	constant eight:	std_logic_vector(7 downto 0) := "00111000";
	constant nine:	std_logic_vector(7 downto 0) := "00111001";
	constant star:	std_logic_vector(7 downto 0) := "00101010";
	constant pound:	std_logic_vector(7 downto 0) := "00100011";
	constant ltr_I: std_logic_vector(7 downto 0) := "01001001";
	constant ltr_G: std_logic_vector(7 downto 0) := "01000111";
	constant ltr_U: std_logic_vector(7 downto 0) := "01010101";
	constant ltr_A: std_logic_vector(7 downto 0) := "01000001";
	constant ltr_S: std_logic_vector(7 downto 0) := "01010011";
--
	-- Some potentially useful puctuation.
	constant SP:    std_logic_vector(7 downto 0) := "00100000"; -- Space
	constant big_delay: integer :=14;
	constant small_delay: integer :=2;
	constant reg_setup: integer :=1;
	constant lastposition: integer:=16;

begin

	lcd_rw<='0';
	process (clock,reset,enable)

		variable position: integer range 0 to lastposition;
		variable count: integer range 0 to big_delay;

	begin

		IF reset = '0' THEN

			state<=initial;
			position:=0;
			count:=0;

		elsIF clock'EVENT AND clock = '1' THEN

			case state is
				WHEN initial => -- to set the function
					if count=reg_setup then
						lcd_enable<='1';
					else
						lcd_enable<='0';
					end if;
					lcd_data<=SET;
					lcd_select<='0';
					if count=small_delay then
						state<=display;
						count:=0;
					else
						count:=count+1;
					end if;

				WHEN display => -- to set display on
					if count=reg_setup then
						lcd_enable<='1';
					else
						lcd_enable<='0';
					end if;
					lcd_data<=DON;
					lcd_select<='0';
					if count=small_delay then
						state<=entrymode;
						count:=0;
					else
						count:=count+1;
					end if;

				WHEN entrymode => -- to set entry mode, increment after each character
								-- is displayed
					if count=reg_setup then
						lcd_enable<='1';
					else
						lcd_enable<='0';
					end if;
					lcd_data<=SEM;
					lcd_select<='0';
					if count=small_delay then
						state<=clear;
						count:=0;
					else
						count:=count+1;
					end if;

				WHEN clear => -- clear the screen
					if count=reg_setup then
						lcd_enable<='1';
					else
						lcd_enable<='0';
					end if;
					lcd_data<=CLR;
					lcd_select<='0';
					if count=big_delay then
						state<=waiting;
						count:=0;
					else
						count:=count+1;
					end if;

				when waiting => -- wait for input
					if enable='0' then
						state<=verify;
					end if;

				when verify =>
					if enable='1' then
						if position=lastposition then
							state<=homecursor;
							position:=0;
						else
							state<=putchar;
						end if;
					end if;

				when putchar=> -- display the character on the LCD
					if count=reg_setup then
						lcd_enable<='1';
					else
						lcd_enable<='0';
					end if;
					case message is
						when "0000" => lcd_data <= zero;
						when "0001" => lcd_data <= one;
						when "0010" => lcd_data <= two;
						when "0011" => lcd_data <= three;
						when "0100" => lcd_data <= four;
						when "0101" => lcd_data <= five;
						when "0110" => lcd_data <= six;
						when "0111" => lcd_data <= seven;
						when "1000" => lcd_data <= eight;
						when "1001" => lcd_data <= nine;
						when "1010" => lcd_data <= star;
						when "1011" => lcd_data <= ltr_I;
						when "1100" => lcd_data <= ltr_G;
						when "1101" => lcd_data <= ltr_U;
						when "1110" => lcd_data <= ltr_A;
						when "1111" => lcd_data <= ltr_S;
						
						when others => lcd_data <= SP;
					end case;
					lcd_select<='1';
					if count=small_delay then
						state<=waiting;
						position:=position+1;
						count:=0;
					else
						count:=count+1;
					end if;

				when homecursor=>
					if count=reg_setup then
						lcd_enable<='1';
					else
						lcd_enable<='0';
					end if;
					lcd_data<=HOME;
					lcd_select<='0';
					if count=big_delay then
						state<=putchar;
						count:=0;
					else
						count:=count+1;
					end if;

			end case;
		end if;
	end process;

end Display;
